library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity PONG is port(
	clk       : in  STD_LOGIC;
    clk_pxl   : in  STD_LOGIC;
    clk_TMDS  : in  STD_LOGIC;
	p1_clk    : out STD_LOGIC;
    p1_rst    : out STD_LOGIC;
    p1_val    : in  STD_LOGIC_VECTOR(3 downto 0);
    p2_clk    : out STD_LOGIC;
    p2_rst    : out STD_LOGIC;
    p2_val    : in  STD_LOGIC_VECTOR(3 downto 0);
    TMDSp     : out STD_LOGIC_VECTOR(2 downto 0);
    TMDSn     : out STD_LOGIC_VECTOR(2 downto 0);
    TMDSp_clk : out STD_LOGIC;
    TMDSn_clk : out STD_LOGIC;
    ADCclk    : in  STD_LOGIC;
    locked    : in  STD_LOGIC
);
end PONG;

architecture behavioral of PONG is

    component TMDS is port(
        clk      : in  STD_LOGIC;
        vd_en    : in  STD_LOGIC;
        ctrl     : in  STD_LOGIC_VECTOR(1 downto 0);
        data_in  : in  STD_LOGIC_VECTOR(7 downto 0);
        data_out : out STD_LOGIC_VECTOR(9 downto 0)
    );
    end component;
    
    component ADC is port(
        clock_clk              : in  std_logic                     := 'X';             -- clk
        reset_sink_reset_n     : in  std_logic                     := 'X';             -- reset_n
        adc_pll_clock_clk      : in  std_logic                     := 'X';             -- clk
        adc_pll_locked_export  : in  std_logic                     := 'X';             -- export
        command_valid          : in  std_logic                     := 'X';             -- valid
        command_channel        : in  std_logic_vector(4 downto 0)  := (others => 'X'); -- channel
        command_startofpacket  : in  std_logic                     := 'X';             -- startofpacket
        command_endofpacket    : in  std_logic                     := 'X';             -- endofpacket
        command_ready          : out std_logic;                                        -- ready
        response_valid         : out std_logic;                                        -- valid
        response_channel       : out std_logic_vector(4 downto 0);                     -- channel
        response_data          : out std_logic_vector(11 downto 0);                    -- data
        response_startofpacket : out std_logic;                                        -- startofpacket
        response_endofpacket   : out std_logic                                         -- endofpacket
    );
	end component ADC;
    
    signal Red             : STD_LOGIC_VECTOR(7 downto 0);
    signal Green           : STD_LOGIC_VECTOR(7 downto 0);
    signal Blue            : STD_LOGIC_VECTOR(7 downto 0);
    signal TMDS_R          : STD_LOGIC_VECTOR(9 downto 0);
    signal TMDS_G          : STD_LOGIC_VECTOR(9 downto 0);
    signal TMDS_B          : STD_LOGIC_VECTOR(9 downto 0);
    signal counterX        : UNSIGNED(9 downto 0);
    signal counterY        : UNSIGNED(9 downto 0);
    signal drawArea        : STD_LOGIC;
    signal hSync           : STD_LOGIC;
    signal vSync           : STD_LOGIC;
    signal TMDS_load       : STD_LOGIC;
    signal TMDS_mod10      : UNSIGNED(3 downto 0);
    signal TMDS_ShiftR     : STD_LOGIC_VECTOR(9 downto 0);
    signal TMDS_ShiftG     : STD_LOGIC_VECTOR(9 downto 0);
    signal TMDS_ShiftB     : STD_LOGIC_VECTOR(9 downto 0);
    signal border          : STD_LOGIC;
    signal ball            : STD_LOGIC;
    signal ballIncX        : STD_LOGIC;
    signal ballIncY        : STD_LOGIC;
    signal ballX           : UNSIGNED(9 downto 0) := to_unsigned(315, 10);
    signal ballY           : UNSIGNED(9 downto 0) := to_unsigned(235, 10);
    signal paddle          : STD_LOGIC;
    signal paddleX         : UNSIGNED(9 downto 0) := to_unsigned(270, 10);
    signal pongCounter     : UNSIGNED(19 downto 0);
    signal points1         : STD_LOGIC;
    signal points2         : STD_LOGIC;
    signal rsp_data        : STD_LOGIC_VECTOR(11 DOWNTO 0);
    signal rsp_valid       : STD_LOGIC;
    signal cmd_endofpacket : STD_LOGIC;
    signal rsp_channel     : STD_LOGIC_VECTOR(4 DOWNTO 0);
    
begin

    p1_rst <= '1';
    p2_rst <= '1';

    R : TMDS port map(
        clk      => clk_pxl,
        vd_en    => drawArea,
        ctrl     => "00",
        data_in  => Red,
        data_out => TMDS_R
    );
    G : TMDS port map(
        clk      => clk_pxl,
        vd_en    => drawArea,
        ctrl     => "00",
        data_in  => Green,
        data_out => TMDS_G
    );
    B : TMDS port map(
        clk      => clk_pxl,
        vd_en    => drawArea,
        ctrl     => vSync & hSync,
        data_in  => Blue,
        data_out => TMDS_B
    );
    
    ADC0 : component ADC port map(
        clock_clk              => clk,                                 --          clock.clk
        reset_sink_reset_n     => '1',                                 --     reset_sink.reset_n
        adc_pll_clock_clk      => ADCclk,                              --  adc_pll_clock.clk
        adc_pll_locked_export  => locked,                              -- adc_pll_locked.export
        command_valid          => '1',                                 --        command.valid
        command_channel        => "00000",                             --               .channel
        command_startofpacket  => '1',                                 --               .startofpacket
        command_endofpacket    => cmd_endofpacket,                     --               .endofpacket
        command_ready          => open,                                --               .ready
        response_valid         => rsp_valid,                           --       response.valid
        response_channel       => open,                                --               .channel
        response_data          => rsp_data,                            --               .data
        response_startofpacket => open,                                --               .startofpacket
        response_endofpacket   => open                                 --               .endofpacket
    );
        
    HDMI1 : process(clk_pxl)
    begin
        if rising_edge(clk_pxl) then
            if counterX < 640 AND counterY < 480 then
                drawArea <= '1';
            else
                drawArea <= '0';
            end if;
            
            if counterX = 799 then
                counterX <= "0000000000";
                if counterY = 524 then
                    counterY <= "0000000000";
                else
                    counterY <= counterY + 1;
                end if;
            else
                counterX <= counterX + 1;
            end if;
            
            if counterX >= 656 AND counterX < 752 then
                hSync <= '1';
            else
                hSync <= '0';
            end if;
            
            if counterY >= 490 AND counterY < 492 then
                vSync <= '1';
            else
                vSync <= '0';
            end if;
        end if;
    end process;
   
    HDMI2 : process(clk_TMDS)
    begin
        if rising_edge(clk_TMDS) then
            if TMDS_mod10 = 9 then
                TMDS_load <= '1';
                TMDS_mod10 <= "0000";
            else
                TMDS_load <= '0';
                TMDS_mod10 <= TMDS_mod10 + 1;
            end if;
            
            if TMDS_load = '1' then
                TMDS_ShiftR <= TMDS_R;
                TMDS_ShiftG <= TMDS_G;
                TMDS_ShiftB <= TMDS_B;
            else
                TMDS_ShiftR <= '0' & TMDS_ShiftR(9 downto 1);
                TMDS_ShiftG <= '0' & TMDS_ShiftG(9 downto 1);
                TMDS_ShiftB <= '0' & TMDS_ShiftB(9 downto 1);
            end if;
            
            
        end if;
    end process;
    
    TMDSp(2) <= TMDS_ShiftR(0);
    TMDSn(2) <= NOT(TMDS_ShiftR(0));
    TMDSp(1) <= TMDS_ShiftG(0);
    TMDSn(1) <= NOT(TMDS_ShiftG(0));
    TMDSp(0) <= TMDS_ShiftB(0);
    TMDSn(0) <= NOT(TMDS_ShiftB(0));
    TMDSp_clk <= clk_pxl;
    TMDSn_clk <= NOT(clk_pxl);

    
       
    
    border <= '1' when counterX < 10 OR counterX >= 629 OR counterY < 10 OR counterY >= 469 else '0';
    ball <= '1' when counterX >= ballX AND counterX < ballX + 10 AND counterY >= ballY AND counterY < ballY + 10 else '0';
    paddle <= '1' when counterX >= paddleX AND counterX < paddleX + 100 AND counterY >= 400 AND counterY < 420 else '0';
    red <= ball & ball & paddle & ball & ball & ball & "11";
    green <= (points1 OR points2) & (points1 OR points2) & paddle & (points1 OR points2) & (points1 OR points2) & (points1 OR points2) & "11";
    blue <= border & border & paddle & border & border & border & "11";
            
    PONG : process(clk, p1_val, p2_val, counterY, counterX)
    begin
        if (unsigned(p1_val) >= 1 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 1))) AND (counterX < (15 + (10 * 1)))) OR
           (unsigned(p1_val) >= 2 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 2))) AND (counterX < (15 + (10 * 2)))) OR
           (unsigned(p1_val) >= 3 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 3))) AND (counterX < (15 + (10 * 3)))) OR
           (unsigned(p1_val) >= 4 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 4))) AND (counterX < (15 + (10 * 4)))) OR
           (unsigned(p1_val) >= 5 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 5))) AND (counterX < (15 + (10 * 5)))) OR
           (unsigned(p1_val) >= 6 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 6))) AND (counterX < (15 + (10 * 6)))) OR
           (unsigned(p1_val) >= 7 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 7))) AND (counterX < (15 + (10 * 7)))) OR
           (unsigned(p1_val) >= 8 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 8))) AND (counterX < (15 + (10 * 8)))) OR
           (unsigned(p1_val) >= 9 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 9))) AND (counterX < (15 + (10 * 9)))) OR
           (unsigned(p1_val) >= 10 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 10))) AND (counterX < (15 + (10 * 10)))) OR
           (unsigned(p1_val) >= 11 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 11))) AND (counterX < (15 + (10 * 11)))) OR
           (unsigned(p1_val) >= 12 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 12))) AND (counterX < (15 + (10 * 12)))) OR
           (unsigned(p1_val) >= 13 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 13))) AND (counterX < (15 + (10 * 13)))) OR
           (unsigned(p1_val) >= 14 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 14))) AND (counterX < (15 + (10 * 14)))) OR
           (unsigned(p1_val) >= 15 AND (counterY > 2) AND (counterY < 8) AND (counterX > (10 + (10 * 15))) AND (counterX < (15 + (10 * 15)))) then
           points1 <= '1';
        else
           points1 <= '0';
        end if;
        
        if (unsigned(p2_val) >= 1 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 1))) AND (counterX < (15 + (10 * 1)))) OR
           (unsigned(p2_val) >= 2 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 2))) AND (counterX < (15 + (10 * 2)))) OR
           (unsigned(p2_val) >= 3 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 3))) AND (counterX < (15 + (10 * 3)))) OR
           (unsigned(p2_val) >= 4 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 4))) AND (counterX < (15 + (10 * 4)))) OR
           (unsigned(p2_val) >= 5 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 5))) AND (counterX < (15 + (10 * 5)))) OR
           (unsigned(p2_val) >= 6 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 6))) AND (counterX < (15 + (10 * 6)))) OR
           (unsigned(p2_val) >= 7 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 7))) AND (counterX < (15 + (10 * 7)))) OR
           (unsigned(p2_val) >= 8 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 8))) AND (counterX < (15 + (10 * 8)))) OR
           (unsigned(p2_val) >= 9 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 9))) AND (counterX < (15 + (10 * 9)))) OR
           (unsigned(p2_val) >= 10 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 10))) AND (counterX < (15 + (10 * 10)))) OR
           (unsigned(p2_val) >= 11 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 11))) AND (counterX < (15 + (10 * 11)))) OR
           (unsigned(p2_val) >= 12 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 12))) AND (counterX < (15 + (10 * 12)))) OR
           (unsigned(p2_val) >= 13 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 13))) AND (counterX < (15 + (10 * 13)))) OR
           (unsigned(p2_val) >= 14 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 14))) AND (counterX < (15 + (10 * 14)))) OR
           (unsigned(p2_val) >= 15 AND (counterY > 471) AND (counterY < 477) AND (counterX > (10 + (10 * 15))) AND (counterX < (15 + (10 * 15)))) then
           points2 <= '1';
        else
           points2 <= '0';
        end if;
        
        if rising_edge(clk) then
            if pongCounter = 50000 then
                if rsp_valid = '1' then
                    if unsigned(rsp_data(11 downto 2)) < 10 then
                        paddleX <= to_unsigned(10, 10);
                    elsif unsigned(rsp_data(11 downto 2)) > 529 then
                        paddleX <= to_unsigned(529, 10);
                    else 
                        paddleX <= unsigned(rsp_data(11 downto 7)) & "00000";
                    end if;                  
                end if;
                
                pongCounter <= (others => '0');
                if ballIncX = '1' AND ballX = 618 then
                    ballIncX <= '0';
                elsif ballIncX = '0' AND ballX = 11 then
                    ballIncX <= '1';
                end if;
                
                
                if ballIncY = '1' AND ballY = 390 AND ballX >= (paddleX - 9) AND ballX < (paddleX + 109) then
                    ballIncY <= '0';
                elsif ballIncY = '1' AND ballY = 458 then
                    ballIncY <= '0';
                    p2_clk <= '1';
                    ballX <= paddleX + 50;
                    ballY <= to_unsigned(235, 10);
                elsif ballIncY = '0' AND ballY = 11 then
                    p1_clk <= '1';
                    ballIncY <= '1';
                end if;
                
                if NOT(ballIncY = '1' AND ballY = 458) then
                    if ballIncX then
                        ballX <= ballX + 1;
                    else 
                        ballX <= ballX - 1;
                    end if;
                    if ballIncY then
                        ballY <= ballY + 1;
                    else
                        ballY <= ballY - 1;
                    end if;
                end if;
            else
                pongCounter <= pongCounter + 1;
                p1_clk <= '0';
                p2_clk <= '0';
            end if;
        end if;
    end process;

end;